unit Keywords;

interface

uses
  Classes, SysUtils;

type
  TKeyword = class(TStringlist)
    constructor Create;
    function IsKeyword (s:string): Boolean;
    procedure SetRange(ext:string);
    procedure Sort; override;
  public
    i1, i2: Integer;
  private
    CaseSensitive: Boolean;
    procedure QuickSort_2(L, R: Integer);
  end;

implementation

{ ------------------------- Methoden fr TKeyword -------------------------- }

constructor TKeyword.Create;
begin
  inherited Create;
  Sorted  := False;
  Duplicates := dupAccept;
  CaseSensitive := True;
  { Voreinstellung: Suchen in der gesamten Liste: }
  i1 := 0;
  i2 := -1;
end;


{ ---------- Testen, ob 's' im Bereich 'i1', 'i2' enthalten ist.
             Dabei wird mit binrer Suche gearbeitet. Die Methode 'IndexOf' von
             TStringList wre viel zu langsam. }

function TKeyword.IsKeyword (s:string): Boolean;
var
  i,j,k,c:Integer;
begin
  { Suchbereich [i,j] festlegen }
  i := i1;
  if i2>=0 then j := i2 else j := Count-1;
  result := False;
  if j<i then
    Exit;
  { Suche initialisieren: Nur anfangen, wenn Strings[i]<s<Strings[j] gilt. }
  if not CaseSensitive then
    s := LowerCase(s);
  c := CompareStr(Strings[i],s);
  if c>=0 then begin
    if c=0 then result := True;
    Exit;
  end;
  c := CompareStr(Strings[j],s);
  if c<=0 then begin
    if c=0 then result := True;
    Exit;
  end;
  { Binre Suche nach 's'. }
  while (j-i>1) and (c<>0) do begin
    k := (i+j) div 2;
    c := CompareStr(Strings[k],s);
    if c<0 then i := k else if c>0 then j := k;
  end;
  result := c=0;
end;


{ ---------- Bereich 'i1', 'i2' anhand der Extenstion setzen.
             Wenn die Stringliste keine Extensions (Strings der Form: '.xxx.yyy.zzz....')
             enthlt, dann wird alles ausgewhlt (i1=0; i2=-1).
             Wenn Extensions enthalten sind, aber 'ext' nicht, dann wird nichts ausgewhlt
             (i1>i2>=0). }

procedure TKeyword.SetRange(ext:string);
begin
  i1 := 0;
  while (i1<Count) and (Pos('.',Strings[i1])<>1) do Inc(i1);
  if i1=Count then begin
    { Liste enthlt keine Extensions }
    i1 := 0;
    i2 := -1;
  end else begin
    ext := LowerCase(ext);
    { Liste enthlt Extensions; jetzt nach 'ext' suchen }
    i1 := 0;
    while (i1<Count) and ((Pos('.',Strings[i1])<>1) or
           ((Pos(ext,Strings[i1])=0) and (Pos(UpperCase(ext),Strings[i1])=0))) do Inc(i1);
    if i1>=Count-1 then
      { 'ext' ist nicht in der Liste (oder im letzten String) }
      i2 := 0
    else begin
      CaseSensitive := Pos(ext,Strings[i1])>0;
      Inc(i1);
      { 'ext' gefunden; jetzt das Ende des Bereichs bestimmen }
      i2 := i1;
      while (i2+1<Count) and (Pos('.',Strings[i2+1])<>1) do Inc(i2);
    end;
  end;
end;


{ ---------- Sortierung der Schlsselwrter
             (Analog TStringlist.Sort, aber mit 'CompareStr' statt 'AnsiCompareText'. }

procedure TKeyword.QuickSort_2(L, R: Integer);
var
  I, J: Integer;
  P: string;
begin
  repeat
    I := L;
    J := R;
    P := Strings[(L + R) shr 1];
    repeat
      while CompareStr(Strings[I], P) < 0 do Inc(I);
      while CompareStr(Strings[J], P) > 0 do Dec(J);
      if I <= J then begin
        Exchange(I, J);
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort_2(L, J);
    L := I;
  until I >= R;
end;


{ ---------- Liste sortieren;
             Dies ist eine Kopie der Methode "TStringlist.Sort" mit "QuickSort_2" anstelle
             von "QuickSort". Wre "TStringlist.QuickSort" virtuell, dann wre es nicht
             ntig gewesen, "Sort" zu berschreiben. }

procedure TKeyword.Sort;
begin
  if not Sorted and (Count > 1) then begin
    Changing;
    QuickSort_2(0, Count - 1);
    Changed;
  end;
end;


end.

